const CLOSE_BUTTON = 2;
const REARRANGE_DELAY = 750;


const Lang = imports.lang;
const St = imports.gi.St;
const Main = imports.ui.main;
const Tweener = imports.ui.tweener;
const Workspace = imports.ui.workspace
const Mainloop = imports.mainloop;

let _regularOnClicked;
let _regularDoRemoveWindow;


function init() {
    _regularOnClicked = Workspace.WindowClone.prototype._onClicked;
    _regularDoRemoveWindow = Workspace.Workspace.prototype._doRemoveWindow;
}

function enable() {
    // save regular versions of functions in prototype to easily make use of
    // them in the new versions (don't know enough of js to know whether there
    // are better options
    Workspace.WindowClone.prototype._regularOnClicked = _regularOnClicked;
    Workspace.Workspace.prototype._regularDoRemoveWindow =
        _regularDoRemoveWindow;

    // override WindowClone's _onClicked
    Workspace.WindowClone.prototype._onClicked = function(action, actor) {
        this._selected = true;
        if (action.get_button() == CLOSE_BUTTON) {
            this.metaWindow.delete(global.get_current_time());
        } else {
            this._regularOnClicked(action, actor);
        }
    };

    // override Workspace's _doRemoveWindow in order to put into it the
    // parameteriseable rearrangement delay. Rather than copy the code from
    // workspace.js, we reuse it but remove the scheduled rearrangement task
    // (as its 750ms delay is hard-coded...)
    Workspace.Workspace.prototype._doRemoveWindow = function(metaWin) {
        this._regularDoRemoveWindow(metaWin);

        // the rest is more or less copied from the tail of _doRemoveWindow's
        // original code

        // remove old handler
        if (this._repositionWindowsId > 0) {
            Mainloop.source_remove(this._repositionWindowsId);
            this._repositionWindowsId = 0;
        }

        // setup new handler
        let [x, y, mask] = global.get_pointer();
        this._cursorX = x;
        this._cursorY = y;

        // this is the bit that changes
        this._repositionWindowsId = Mainloop.timeout_add(REARRANGE_DELAY,
                            Lang.bind(this, this._delayedWindowRepositioning));
    }
}

function disable() {
    Workspace.WindowClone.prototype._onClicked = _regularOnClicked;
    Workspace.Workspace.prototype._doRemoveWindow = _regularDoRemoveWindow;
    delete Workspace.WindowClone.prototype._regularOnClicked;
    delete Workspace.WindowClone.prototype._regularDoRemoveWindow;
}
